@@ -1,9 +1,3 @@
-use std::ffi::CString;
-
-pub unsafe fn new_c_str(s: &str) -> CString {
-    return CString::new(s).unwrap();
-}
-
 #[macro_export]
 macro_rules! c_str {
     ($s:literal) => {
