@@ -9,7 +9,13 @@
 /// 6. Box - errors, sessions, sequences, transactions, indexes, spaces, tuples.
 use std::os::raw::{c_char, c_int, c_uint, c_void};
 
-use va_list::VaList;
+use bitflags::bitflags;
+
+#[cfg(not(all(target_arch = "aarch64", target_os = "macos")))]
+use ::va_list::VaList;
+
+#[cfg(all(target_arch = "aarch64", target_os = "macos"))]
+use crate::va_list::VaList;
 
 // Clock.
 extern "C" {
@@ -86,16 +92,16 @@ extern "C" {
     ///
     /// # Example
     /// ```c
-    ///	static ssize_t openfile_cb(va_list ap)
-    ///	{
-    ///	         const char *filename = va_arg(ap);
-    ///	         int flags = va_arg(ap);
-    ///	         return open(filename, flags);
-    ///	}
+    /// static ssize_t openfile_cb(va_list ap)
+    /// {
+    ///      const char *filename = va_arg(ap);
+    ///      int flags = va_arg(ap);
+    ///      return open(filename, flags);
+    /// }
     ///
-    ///	if (coio_call(openfile_cb, "/tmp/file", 0) == -1)
-    ///		// handle errors.
-    ///	...
+    /// if (coio_call(openfile_cb, "/tmp/file", 0) == -1)
+    ///     // handle errors.
+    /// ...
     /// ```
     pub fn coio_call(func: Option<unsafe extern "C" fn(VaList) -> c_int>, ...) -> isize;
 }
@@ -424,8 +430,8 @@ extern "C" {
     /// - `0`: success
     /// - `-1`: failure
     ///     - errno=ETIMEDOUT if timeout exceeded,
-    ///	    - errno=ECANCEL if the fiber is cancelled
-    ///	    - errno=EBADF if the channel is closed while waiting on it.
+    ///     - errno=ECANCEL if the fiber is cancelled
+    ///     - errno=EBADF if the channel is closed while waiting on it.
     ///
     pub fn fiber_channel_put_msg_timeout(
         ch: *mut fiber_channel,
@@ -454,29 +460,40 @@ extern "C" {
 }
 
 /// Channel buffer size.
+///
+/// # Safety
+/// `ch` must point to a valid instance of [`fiber_channel`]
 #[inline(always)]
 pub unsafe fn fiber_channel_size(ch: *mut fiber_channel) -> u32 {
-    (&*ch).size
+    (*ch).size
 }
 
 /// The number of messages in the buffer.
 /// There may be more messages outstanding
 /// if the buffer is full.
+///
+/// # Safety
+/// `ch` must point to a valid instance of [`fiber_channel`]
 #[inline(always)]
 pub unsafe fn fiber_channel_count(ch: *mut fiber_channel) -> u32 {
-    (&*ch).count
+    (*ch).count
 }
 
 /// True if the channel is closed for both for reading
 /// and writing.
+///
+/// # Safety
+/// `ch` must point to a valid instance of [`fiber_channel`]
 #[inline(always)]
 pub unsafe fn fiber_channel_is_closed(ch: *mut fiber_channel) -> bool {
-    (&*ch).is_closed
+    (*ch).is_closed
 }
 
+/// # Safety
+/// `ch` must point to a valid instance of [`fiber_channel`]
 #[inline(always)]
 pub unsafe fn fiber_channel_is_empty(ch: *mut fiber_channel) -> bool {
-    (&*ch).count == 0
+    (*ch).count == 0
 }
 
 #[repr(C)]
@@ -742,7 +759,45 @@ extern "C" {
 
 #[repr(C)]
 pub struct BoxTuple {
-    _unused: [u8; 0],
+    refs: u16,
+    format_id: u16,
+    bsize: u32,
+    data_offset: u16,
+}
+
+impl BoxTuple {
+    /// # Safety
+    /// Access to a field of a struct that can be changed in a future version of
+    /// tarantool. Valid for 2.9.0
+    #[inline(always)]
+    pub unsafe fn bsize(&self) -> u32 {
+        self.bsize
+    }
+
+    /// # Safety
+    /// Access to a field of a struct that can be changed in a future version of
+    /// tarantool. Valid for 2.9.0
+    #[inline(always)]
+    pub unsafe fn data_offset(&self) -> u16 {
+        // The last bit is a `is_dirty` flag since 2.5.1
+        self.data_offset & (u16::MAX >> 1)
+    }
+
+    /// # Safety
+    /// Access to a field of a struct that can be changed in a future version of
+    /// tarantool. Valid for 2.9.0
+    #[inline(always)]
+    pub unsafe fn data(&self) -> *const c_char {
+        (self as *const Self).cast::<c_char>().add(self.data_offset() as _)
+    }
+
+    /// # Safety
+    /// Access to a field of a struct that can be changed in a future version of
+    /// tarantool. Valid for 2.9.0
+    #[inline(always)]
+    pub unsafe fn field_map(&self) -> *const u32 {
+        self.data() as _
+    }
 }
 
 #[repr(C)]
@@ -781,6 +836,15 @@ extern "C" {
         key_b: *const c_char,
         key_def: *mut BoxKeyDef,
     ) -> c_int;
+
+    pub fn tuple_field_raw_by_full_path(
+        format: *const BoxTupleFormat,
+        tuple: *const c_char,
+        field_map: *const u32,
+        path: *const c_char,
+        path_len: u32,
+        path_hash: u32,
+    ) -> *const c_char;
 }
 
 #[repr(C)]
