@@ -16,8 +16,13 @@ use std::os::raw::c_void;
 use std::ptr::NonNull;
 use std::time::Duration;
 
-use crate::tlua::{self as tlua, AsLua, lua_error};
-use va_list::VaList;
+use crate::tlua::{self as tlua, AsLua};
+
+#[cfg(not(all(target_arch = "aarch64", target_os = "macos")))]
+use ::va_list::{VaList, VaPrimitive};
+
+#[cfg(all(target_arch = "aarch64", target_os = "macos"))]
+use crate::va_list::{VaList, VaPrimitive};
 
 use crate::c_ptr;
 use crate::error::TarantoolError;
@@ -30,6 +35,41 @@ pub use channel::{
     Channel, SendTimeout, RecvTimeout, SendError, RecvError, TrySendError, TryRecvError,
 };
 
+pub mod mutex;
+pub use mutex::Mutex;
+
+macro_rules! impl_debug_stub {
+    ($t:ident $($p:tt)*) => {
+        impl $($p)* ::std::fmt::Debug for $t $($p)* {
+            fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
+                f.debug_struct(::std::stringify!($t))
+                    .finish_non_exhaustive()
+            }
+        }
+    }
+}
+
+macro_rules! impl_eq_hash {
+    ($t:ident $($p:tt)*) => {
+        impl $($p)* ::std::cmp::PartialEq for $t $($p)* {
+            fn eq(&self, other: &Self) -> bool {
+                self.inner == other.inner
+            }
+        }
+
+        impl $($p)* ::std::cmp::Eq for $t $($p)* {}
+
+        impl $($p)* ::std::hash::Hash for $t $($p)* {
+            fn hash<H>(&self, state: &mut H)
+            where
+                H: ::std::hash::Hasher,
+            {
+                self.inner.hash(state)
+            }
+        }
+    }
+}
+
 /// *OBSOLETE*: This struct is being deprecated in favour of [`Immediate`],
 /// [`Deferred`], etc. due to them being more efficient and idiomatic.
 ///
@@ -74,6 +114,8 @@ pub struct Fiber<'a, T: 'a> {
     phantom: PhantomData<&'a T>,
 }
 
+impl_debug_stub!{Fiber<'a, T>}
+
 impl<'a, T> Fiber<'a, T> {
     /// Create a new fiber.
     ///
@@ -215,6 +257,8 @@ pub struct Builder<F> {
     f: F,
 }
 
+impl_debug_stub!{Builder<F>}
+
 impl Builder<NoFunc> {
     /// Generates the base configuration for spawning a fiber, from which
     /// configuration methods can be chained.
@@ -227,10 +271,10 @@ impl Builder<NoFunc> {
     }
 
     /// Sets the callee function for the new fiber.
-    pub fn func<F, T>(self, f: F) -> Builder<FiberFunc<F, T>>
+    pub fn func<'f, F, T>(self, f: F) -> Builder<FiberFunc<'f, F, T>>
     where
         F: FnOnce() -> T,
-        F: 'static,
+        F: 'f,
     {
         Builder {
             name: self.name,
@@ -238,24 +282,34 @@ impl Builder<NoFunc> {
             f: FiberFunc {
                 f: Box::new(f),
                 result: Default::default(),
+                marker: PhantomData,
             },
         }
     }
 
     /// Sets the callee function for the new fiber.
-    pub fn proc<F>(self, f: F) -> Builder<FiberProc<F>>
+    pub fn proc<'f, F>(self, f: F) -> Builder<FiberProc<'f, F>>
     where
         F: FnOnce(),
-        F: 'static,
+        F: 'f,
     {
         Builder {
             name: self.name,
             attr: self.attr,
-            f: FiberProc { f: Box::new(f) },
+            f: FiberProc {
+                f: Box::new(f),
+                marker: PhantomData,
+            },
         }
     }
 }
 
+impl Default for Builder<NoFunc> {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl<F> Builder<F> {
     /// Names the fiber-to-be.
     ///
@@ -345,6 +399,8 @@ pub struct Fyber<C, I> {
     _invocation: PhantomData<I>,
 }
 
+impl_debug_stub!{Fyber<C, I>}
+
 impl<C, I> Fyber<C, I>
 where
     C: Callee,
@@ -426,6 +482,8 @@ pub struct LuaFiber<C> {
     callee: C,
 }
 
+impl_debug_stub!{LuaFiber<C>}
+
 /// Deferred non-yielding fiber implemented using **lua** api. This (hopefully)
 /// temporary implementation is a workaround. Tarantool C API lacks the method
 /// for passing the necessary information into the underlying `struct fiber`
@@ -448,7 +506,7 @@ where
             lua::lua_pushstring(l, c_ptr!("fiber"));
             impl_details::guarded_pcall(l, 1, 1)?;
             lua::lua_getfield(l, -1, c_ptr!("new"));
-            tlua::push_some_userdata(l, callee.into_inner());
+            impl_details::push_userdata(l, callee.into_inner());
             lua::lua_pushcclosure(l, Self::trampoline, 1);
             impl_details::guarded_pcall(l, 1, 1)
                 .map_err(|e| {
@@ -478,13 +536,13 @@ where
         let f = (ud_ptr as *mut Option<C::Function>).as_mut()
             .unwrap_or_else(||
                 // lua_touserdata returned NULL
-                lua_error!(l, "failed to extract upvalue")
+                tlua::error!(l, "failed to extract upvalue")
             )
             // put None back into userdata
             .take()
             .unwrap_or_else(||
                 // userdata originally contained None
-                lua_error!(l, "rust FnOnce callback was called more than once")
+                tlua::error!(l, "rust FnOnce callback was called more than once")
             );
 
         // call f and drop it afterwards
@@ -499,12 +557,15 @@ where
 /// LuaJoinHandle
 ////////////////////////////////////////////////////////////////////////////////
 
-pub struct LuaJoinHandle<T> {
+#[derive(PartialEq, Eq, Hash)]
+pub struct LuaJoinHandle<'f, T> {
     fiber_ref: Option<i32>,
-    marker: PhantomData<T>,
+    marker: PhantomData<(&'f (), T)>,
 }
 
-impl<T> LuaJoinHandle<T> {
+impl_debug_stub!{LuaJoinHandle<'f, T>}
+
+impl<'f, T> LuaJoinHandle<'f, T> {
     fn new(fiber_ref: i32) -> Self {
         Self { fiber_ref: Some(fiber_ref), marker: PhantomData }
     }
@@ -527,7 +588,7 @@ impl<T> LuaJoinHandle<T> {
     }
 }
 
-impl<T> Drop for LuaJoinHandle<T> {
+impl<'f, T> Drop for LuaJoinHandle<'f, T> {
     fn drop(&mut self) {
         if self.fiber_ref.is_some() {
             panic!("LuaJoinHandle dropped before being joined")
@@ -539,13 +600,20 @@ impl<T> Drop for LuaJoinHandle<T> {
 /// LuaUnitJoinHandle
 ////////////////////////////////////////////////////////////////////////////////
 
-pub struct LuaUnitJoinHandle {
+#[derive(PartialEq, Eq, Hash)]
+pub struct LuaUnitJoinHandle<'f> {
     fiber_ref: Option<i32>,
+    marker: PhantomData<&'f ()>,
 }
 
-impl LuaUnitJoinHandle {
+impl_debug_stub!{LuaUnitJoinHandle<'f>}
+
+impl<'f> LuaUnitJoinHandle<'f> {
     fn new(fiber_ref: i32) -> Self {
-        Self { fiber_ref: Some(fiber_ref) }
+        Self {
+            fiber_ref: Some(fiber_ref),
+            marker: PhantomData,
+        }
     }
 
     pub fn join(mut self) {
@@ -557,7 +625,7 @@ impl LuaUnitJoinHandle {
     }
 }
 
-impl Drop for LuaUnitJoinHandle {
+impl<'f> Drop for LuaUnitJoinHandle<'f> {
     fn drop(&mut self) {
         if self.fiber_ref.is_some() {
             panic!("LuaUnitJoinHandle dropped before being joined")
@@ -571,7 +639,7 @@ impl Drop for LuaUnitJoinHandle {
 
 mod impl_details {
     use super::*;
-    use crate::tlua::{AsLua, Lua, LuaError, PushGuard};
+    use crate::tlua::{AsLua, StaticLua, LuaError, PushGuard};
 
     pub(super) unsafe fn lua_error_from_top(l: *mut lua::lua_State) -> LuaError {
         let mut len = std::mem::MaybeUninit::uninit();
@@ -581,7 +649,7 @@ mod impl_details {
             data as *mut u8, len.assume_init()
         );
         let msg = String::from_utf8_lossy(msg_bytes);
-        tlua::LuaError::ExecutionError(msg.into()).into()
+        tlua::LuaError::ExecutionError(msg)
     }
 
     /// In case of success, the stack contains the results.
@@ -603,8 +671,8 @@ mod impl_details {
         }
     }
 
-    pub(super) unsafe fn lua_fiber_join(f_ref: i32) -> Result<PushGuard<Lua>> {
-        let l = Lua::from_existing_state(ffi::luaT_state(), false);
+    pub(super) unsafe fn lua_fiber_join(f_ref: i32) -> Result<PushGuard<StaticLua>> {
+        let l = crate::global_lua();
         let lptr = l.as_lua();
         lua::lua_rawgeti(lptr, lua::LUA_REGISTRYINDEX, f_ref);
         lua::lua_getfield(lptr, -1, c_ptr!("join"));
@@ -647,6 +715,43 @@ mod impl_details {
     //     lua::lua_settop(lptr, top_before);
     //     res
     // }
+
+    /// # Safety
+    /// **WARNING** this function is super unsafe in case `T` is not 'static.
+    /// It's used to implement non-static fibers which is safe because the
+    /// lifetime of `T` is captured in the join handle and so the compiler will
+    /// make sure the fiber is joined before the referenced data is dropped.
+    /// Keep this in mind if you want to use this function
+    pub(super) unsafe fn push_userdata<T>(lua: tlua::LuaState, value: T) {
+        use tlua::ffi;
+        type UDBox<T> = Option<T>;
+        let ud_ptr = ffi::lua_newuserdata(lua, std::mem::size_of::<UDBox<T>>());
+        std::ptr::write(ud_ptr.cast::<UDBox<T>>(), Some(value));
+
+        if std::mem::needs_drop::<T>() {
+            // Creating a metatable.
+            ffi::lua_newtable(lua);
+
+            // Index "__gc" in the metatable calls the object's destructor.
+            ffi::lua_pushstring(lua, c_ptr!("__gc"));
+            ffi::lua_pushcfunction(lua, wrap_gc::<T>);
+            ffi::lua_settable(lua, -3);
+
+            ffi::lua_setmetatable(lua, -2);
+        }
+
+        /// A callback for the "__gc" event. It checks if the value was moved out
+        /// and if not it drops the value.
+        unsafe extern "C" fn wrap_gc<T>(lua: *mut ffi::lua_State) -> i32 {
+            let ud_ptr = ffi::lua_touserdata(lua, 1);
+            let ud = ud_ptr.cast::<UDBox<T>>()
+                .as_mut()
+                .expect("__gc called with userdata pointing to NULL");
+            drop(ud.take());
+
+            0
+        }
+    }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -655,7 +760,7 @@ mod impl_details {
 
 pub trait LuaCallee {
     /// Type of the callee
-    type Function: FnOnce() -> Self::Output + 'static;
+    type Function: FnOnce() -> Self::Output;
 
     /// Return type of the callee
     type Output;
@@ -672,8 +777,9 @@ pub trait LuaCallee {
     /// This function is called within `LuaFiber::trampoline` to save the
     /// return value after the callee's invocation
     ///
-    /// This function is unsafe, because it is very easy to mess things up
-    /// when preparing arugments.
+    /// # Safety
+    /// The function must return the number of values that it has put onto the
+    /// stack as result values
     unsafe fn save_result(l: *mut lua::lua_State, res: Self::Output) -> i32;
 }
 
@@ -681,19 +787,32 @@ pub trait LuaCallee {
 /// LuaFiberFunc
 ////////////////////////////////////////////////////////////////////////////////
 
-pub struct LuaFiberFunc<F>(pub F);
+pub struct LuaFiberFunc<'f, F> {
+    f: F,
+    marker: PhantomData<&'f ()>,
+}
 
-impl<F, T> LuaCallee for LuaFiberFunc<F>
+impl<'f, F> LuaFiberFunc<'f, F> {
+    pub fn new(f: F) -> Self {
+        Self {
+            f,
+            marker: PhantomData,
+        }
+    }
+}
+
+impl<'f, F, T> LuaCallee for LuaFiberFunc<'f, F>
 where
     F: FnOnce() -> T,
-    F: 'static,
+    F: 'f,
+    T: 'f,
 {
     type Function = F;
     type Output = T;
-    type JoinHandle = LuaJoinHandle<T>;
+    type JoinHandle = LuaJoinHandle<'f, T>;
 
     fn into_inner(self) -> F {
-        self.0
+        self.f
     }
 
     fn join_handle(fiber_ref: i32) -> Self::JoinHandle {
@@ -701,7 +820,7 @@ where
     }
 
     unsafe fn save_result(l: *mut lua::lua_State, res: T) -> i32 {
-        tlua::push_some_userdata(l, res);
+        impl_details::push_userdata(l, res);
         1
     }
 }
@@ -710,23 +829,35 @@ where
 /// LuaFiberProc
 ////////////////////////////////////////////////////////////////////////////////
 
-pub struct LuaFiberProc<F>(pub F);
+pub struct LuaFiberProc<'f, F> {
+    f: F,
+    marker: PhantomData<&'f ()>,
+}
 
-impl<F> LuaCallee for LuaFiberProc<F>
+impl<'f, F> LuaFiberProc<'f, F> {
+    pub fn new(f: F) -> Self {
+        Self {
+            f,
+            marker: PhantomData,
+        }
+    }
+}
+
+impl<'f, F> LuaCallee for LuaFiberProc<'f, F>
 where
     F: FnOnce(),
-    F: 'static,
+    F: 'f,
 {
     type Function = F;
     type Output = ();
-    type JoinHandle = LuaUnitJoinHandle;
+    type JoinHandle = LuaUnitJoinHandle<'f>;
 
     fn join_handle(fiber_ref: i32) -> Self::JoinHandle {
         LuaUnitJoinHandle::new(fiber_ref)
     }
 
     fn into_inner(self) -> F {
-        self.0
+        self.f
     }
 
     unsafe fn save_result(_: *mut lua::lua_State, _: ()) -> i32 {
@@ -757,6 +888,7 @@ pub trait Callee {
     /// This function is called within [`Fyber::spawn`] to prepare the arguments
     /// for and invoke the [`ffi::fiber_start`] function.
     ///
+    /// # Safety
     /// This function is unsafe, because it is very easy to mess things up
     /// when preparing arugments.
     unsafe fn start_fiber(self, inner: NonNull<ffi::Fiber>) -> Self::JoinHandle;
@@ -764,6 +896,7 @@ pub trait Callee {
     /// This function is called within `Fyber::trampoline` to extract the
     /// arguments from the [`va_list::VaList`].
     ///
+    /// # Safety
     /// This function is unsafe, because it is very easy to mess things up
     /// when extracting arugments.
     unsafe fn parse_args(args: VaList) -> Self::Args;
@@ -771,6 +904,7 @@ pub trait Callee {
     /// This function is called within `Fyber::trampoline` to invoke the
     /// underlying callee function and process it's results.
     ///
+    /// # Safety
     /// This function is unsafe, because it is very easy to mess things up
     /// when storing the callee's result values.
     unsafe fn invoke(a: Self::Args);
@@ -782,19 +916,23 @@ pub struct NoFunc;
 
 /// This is a helper type used to configure [`Fyber`] with the appropriate
 /// behavior for the fiber function that returns a value.
-pub struct FiberFunc<F, T>
+pub struct FiberFunc<'f, F, T>
 where
     F: FnOnce() -> T,
+    F: 'f,
 {
     f: Box<F>,
     result: Box<UnsafeCell<Option<T>>>,
+    marker: PhantomData<&'f ()>,
 }
 
-impl<F, T> Callee for FiberFunc<F, T>
+impl<'f, F, T> Callee for FiberFunc<'f, F, T>
 where
     F: FnOnce() -> T,
+    F: 'f,
+    T: 'f,
 {
-    type JoinHandle = JoinHandle<T>;
+    type JoinHandle = JoinHandle<'f, T>;
     type Args = (Box<F>, *mut Option<T>);
 
     unsafe fn start_fiber(self, inner: NonNull<ffi::Fiber>) -> Self::JoinHandle {
@@ -816,18 +954,20 @@ where
 
 /// This is a helper type used to configure [`Fyber`] with the appropriate
 /// behavior for the fiber procedure (function which doens't return a value).
-pub struct FiberProc<F>
+pub struct FiberProc<'f, F>
 where
     F: FnOnce(),
 {
     f: Box<F>,
+    marker: PhantomData<&'f ()>,
 }
 
-impl<F> Callee for FiberProc<F>
+impl<'f, F> Callee for FiberProc<'f, F>
 where
     F: FnOnce(),
+    F: 'f,
 {
-    type JoinHandle = UnitJoinHandle;
+    type JoinHandle = UnitJoinHandle<'f>;
     type Args = Box<F>;
 
     unsafe fn start_fiber(self, inner: NonNull<ffi::Fiber>) -> Self::JoinHandle {
@@ -861,6 +1001,7 @@ pub trait Invocation {
     /// This method is called from the `Fyber::trampoline` function right
     /// before calling the fiber function.
     ///
+    /// # Safety
     /// This is an implementation detail and will most likely be removed in the
     /// future.
     unsafe fn before_callee();
@@ -868,6 +1009,7 @@ pub trait Invocation {
     /// This method is called from the [`Fyber::spawn`] function right
     /// after starting the fiber.
     ///
+    /// # Safety
     /// This is an implementation detail and will most likely be removed in the
     /// future.
     unsafe fn after_start(f: NonNull<ffi::Fiber>);
@@ -897,14 +1039,22 @@ impl Invocation for Deferred {
 ////////////////////////////////////////////////////////////////////////////////
 
 /// An owned permission to join on an immediate fiber (block on its termination).
-pub struct JoinHandle<T> {
+pub struct JoinHandle<'f, T> {
     inner: Option<NonNull<ffi::Fiber>>,
     result: Box<UnsafeCell<Option<T>>>,
+    marker: PhantomData<&'f ()>,
 }
 
-impl<T> JoinHandle<T> {
+impl_debug_stub!{JoinHandle<'f, T>}
+impl_eq_hash!{JoinHandle<'f, T>}
+
+impl<'f, T> JoinHandle<'f, T> {
     fn new(inner: NonNull<ffi::Fiber>, result: Box<UnsafeCell<Option<T>>>) -> Self {
-        Self { inner: Some(inner), result }
+        Self {
+            inner: Some(inner),
+            result,
+            marker: PhantomData,
+        }
     }
 
     /// Block until the fiber's termination and return it's result value.
@@ -918,7 +1068,7 @@ impl<T> JoinHandle<T> {
     }
 }
 
-impl<T> Drop for JoinHandle<T> {
+impl<'f, T> Drop for JoinHandle<'f, T> {
     fn drop(&mut self) {
         if self.inner.is_some() {
             panic!("JoinHandle dropped before being joined")
@@ -933,13 +1083,20 @@ impl<T> Drop for JoinHandle<T> {
 /// An owned permission to join on an immediate fiber (block on its termination).
 ///
 /// This is an optimized case of [`JoinHandle`]`<()>`.
-pub struct UnitJoinHandle {
+pub struct UnitJoinHandle<'f> {
     inner: Option<NonNull<ffi::Fiber>>,
+    marker: PhantomData<&'f ()>,
 }
 
-impl UnitJoinHandle {
+impl_debug_stub!{UnitJoinHandle<'f>}
+impl_eq_hash!{UnitJoinHandle<'f>}
+
+impl<'f> UnitJoinHandle<'f> {
     fn new(inner: NonNull<ffi::Fiber>) -> Self {
-        Self { inner: Some(inner) }
+        Self {
+            inner: Some(inner),
+            marker: PhantomData,
+        }
     }
 
     /// Block until the fiber's termination.
@@ -951,7 +1108,7 @@ impl UnitJoinHandle {
     }
 }
 
-impl Drop for UnitJoinHandle {
+impl<'f> Drop for UnitJoinHandle<'f> {
     fn drop(&mut self) {
         if self.inner.is_some() {
             panic!("UnitJoinHandle dropped before being joined")
@@ -968,7 +1125,7 @@ impl Drop for UnitJoinHandle {
 trait TrampolineArgs {
     unsafe fn get<T>(&mut self) -> T
     where
-        T: va_list::VaPrimitive;
+        T: VaPrimitive;
 
     unsafe fn get_boxed<T>(&mut self) -> Box<T> {
         Box::from_raw(self.get::<*const c_void>() as *mut T)
@@ -989,7 +1146,7 @@ trait TrampolineArgs {
 impl TrampolineArgs for VaList {
     unsafe fn get<T>(&mut self) -> T
     where
-        T: va_list::VaPrimitive,
+        T: VaPrimitive,
     {
         self.get::<T>()
     }
@@ -1013,10 +1170,11 @@ impl TrampolineArgs for VaList {
 /// This will create a fiber using default parameters of [`Builder`], if you
 /// want to specify the stack size or the name of the thread, use this API
 /// instead.
-pub fn start<F, T>(f: F) -> JoinHandle<T>
+pub fn start<'f, F, T>(f: F) -> JoinHandle<'f, T>
 where
     F: FnOnce() -> T,
-    F: 'static,
+    F: 'f,
+    T: 'f,
 {
     Builder::new().func(f).start().unwrap()
 }
@@ -1030,10 +1188,10 @@ where
 /// should always be used instead of the latter.
 ///
 /// For more details see: [`start`]
-pub fn start_proc<F>(f: F) -> UnitJoinHandle
+pub fn start_proc<'f, F>(f: F) -> UnitJoinHandle<'f>
 where
     F: FnOnce(),
-    F: 'static,
+    F: 'f,
 {
     Builder::new().proc(f).start().unwrap()
 }
@@ -1049,12 +1207,13 @@ where
 ///
 /// The new fiber can be joined by calling [`LuaJoinHandle::join`] method on
 /// it's join handle.
-pub fn defer<F, T>(f: F) -> LuaJoinHandle<T>
+pub fn defer<'f, F, T>(f: F) -> LuaJoinHandle<'f, T>
 where
     F: FnOnce() -> T,
-    F: 'static,
+    F: 'f,
+    T: 'f,
 {
-    LuaFiber::new(LuaFiberFunc(f)).spawn().unwrap()
+    LuaFiber::new(LuaFiberFunc::new(f)).spawn().unwrap()
 }
 
 /// Creates a new proc fiber and schedules it for execution, returning a
@@ -1067,12 +1226,12 @@ where
 /// it's join handle.
 ///
 /// This is an optimized version [`defer`]`<F, ()>`.
-pub fn defer_proc<F>(f: F) -> LuaUnitJoinHandle
+pub fn defer_proc<'f, F>(f: F) -> LuaUnitJoinHandle<'f>
 where
     F: FnOnce(),
-    F: 'static,
+    F: 'f,
 {
-    LuaFiber::new(LuaFiberProc(f)).spawn().unwrap()
+    LuaFiber::new(LuaFiberProc::new(f)).spawn().unwrap()
 }
 
 /// Make it possible or not possible to wakeup the current
@@ -1137,6 +1296,7 @@ pub fn reschedule() {
 }
 
 /// Fiber attributes container
+#[derive(Debug)]
 pub struct FiberAttr {
     inner: *mut ffi::FiberAttr,
 }
@@ -1171,6 +1331,12 @@ impl FiberAttr {
     }
 }
 
+impl Default for FiberAttr {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl Drop for FiberAttr {
     fn drop(&mut self) {
         unsafe { ffi::fiber_attr_delete(self.inner) }
@@ -1205,6 +1371,7 @@ impl Drop for FiberAttr {
 /// In real life, programmers would make sure to use different conditional variable names for different applications.
 ///
 /// Unlike `pthread_cond`, [Cond]() doesn't require mutex/latch wrapping.
+#[derive(Debug)]
 pub struct Cond {
     inner: *mut ffi::FiberCond,
 }
@@ -1246,12 +1413,20 @@ impl Cond {
     /// - `true` on [signal()](#method.signal) call or a spurious wake up.
     /// - `false` on timeout, diag is set to `TimedOut`
     pub fn wait_timeout(&self, timeout: Duration) -> bool {
-        !(unsafe { ffi::fiber_cond_wait_timeout(self.inner, timeout.as_secs_f64()) } < 0)
+        unsafe {
+            ffi::fiber_cond_wait_timeout(self.inner, timeout.as_secs_f64()) >= 0
+        }
     }
 
     /// Shortcut for [wait_timeout()](#method.wait_timeout).
     pub fn wait(&self) -> bool {
-        !(unsafe { ffi::fiber_cond_wait(self.inner) } < 0)
+        unsafe { ffi::fiber_cond_wait(self.inner) >= 0 }
+    }
+}
+
+impl Default for Cond {
+    fn default() -> Self {
+        Self::new()
     }
 }
 
@@ -1262,6 +1437,7 @@ impl Drop for Cond {
 }
 
 /// A lock for cooperative multitasking environment
+#[derive(Debug)]
 pub struct Latch {
     inner: *mut ffi::Latch,
 }
@@ -1298,6 +1474,12 @@ impl Latch {
     }
 }
 
+impl Default for Latch {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl Drop for Latch {
     fn drop(&mut self) {
         unsafe { ffi::box_latch_delete(self.inner) }
@@ -1306,6 +1488,7 @@ impl Drop for Latch {
 
 /// An RAII implementation of a "scoped lock" of a latch. When this structure is dropped (falls out of scope),
 /// the lock will be unlocked.
+#[derive(Debug)]
 pub struct LatchGuard {
     latch_inner: *mut ffi::Latch,
 }
