@@ -16,6 +16,7 @@ use crate::error::{Error, TarantoolErrorCode};
 use crate::fiber::Cond;
 use crate::net_box::{Conn, ConnOptions, Options};
 use crate::raft::inner::NodeEvent;
+use crate::set_error;
 use crate::tuple::{FunctionArgs, FunctionCtx, Tuple};
 
 mod fsm;
@@ -139,10 +140,9 @@ impl Node {
     }
 
     pub fn wait_ready(&self, timeout: Duration) -> Result<(), Error> {
-        if self.inner.borrow().state() != &NodeState::Ready {
-            if !self.ready_cond.wait_timeout(timeout) {
-                return Err(Error::IO(io::ErrorKind::TimedOut.into()));
-            }
+        if self.inner.borrow().state() != &NodeState::Ready
+            && !self.ready_cond.wait_timeout(timeout) {
+            return Err(Error::IO(io::ErrorKind::TimedOut.into()));
         }
         Ok(())
     }
