@@ -1,4 +1,4 @@
-#![cfg(feature = "schema")]
+#![cfg(any(feature = "schema", doc))]
 
 pub mod index;
 pub mod sequence;
